@@ -1,7 +1,7 @@
 /*
  * This file is part of nzbget. See <http://nzbget.net>.
  *
- * Copyright (C) 2012-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * Copyright (C) 2012-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,12 @@
 /*
  * In this module:
  *   1) Download tab;
- *   2) Functions for html generation for downloads, also used from other modules (edit and add dialogs).
+ *   2) Functions for html generation for downloads, also used from other modules (edit and add dialogs);
+ *   3) Popup menus in downloads list.
  */
 
 /*** DOWNLOADS TAB ***********************************************************/
- 
+
 var Downloads = (new function($)
 {
 	'use strict';
@@ -36,6 +37,8 @@ var Downloads = (new function($)
 	var $DownloadQueueEmpty;
 	var $DownloadsRecordsPerPage;
 	var $DownloadsTable_Name;
+	var $PriorityMenu;
+	var $CategoryMenu;
 
 	// State
 	var notification = null;
@@ -43,7 +46,8 @@ var Downloads = (new function($)
 	var groups;
 	var urls;
 	var nameColumnWidth = null;
-	var minLevel = null;
+	var cached = false;
+	var lastDownloadRate;
 
 	var statusData = {
 		'QUEUED': { Text: 'QUEUED', PostProcess: false },
@@ -75,6 +79,8 @@ var Downloads = (new function($)
 		$DownloadQueueEmpty = $('#DownloadQueueEmpty');
 		$DownloadsRecordsPerPage = $('#DownloadsRecordsPerPage');
 		$DownloadsTable_Name = $('#DownloadsTable_Name');
+		$PriorityMenu = $('#PriorityMenu');
+		$CategoryMenu = $('#DownloadsCategoryMenu');
 
 		var recordsPerPage = UISettings.read('DownloadsRecordsPerPage', 10);
 		$DownloadsRecordsPerPage.val(recordsPerPage);
@@ -82,49 +88,60 @@ var Downloads = (new function($)
 
 		$DownloadsTable.fasttable(
 			{
-				filterInput: $('#DownloadsTable_filter'),
-				filterClearButton: $("#DownloadsTable_clearfilter"),
-				pagerContainer: $('#DownloadsTable_pager'),
-				infoContainer: $('#DownloadsTable_info'),
-				headerCheck: $('#DownloadsTable > thead > tr:first-child'),
+				filterInput: '#DownloadsTable_filter',
+				filterClearButton: '#DownloadsTable_clearfilter',
+				pagerContainer: '#DownloadsTable_pager',
+				infoContainer: '#DownloadsTable_info',
 				infoEmpty: '&nbsp;', // this is to disable default message "No records"
 				pageSize: recordsPerPage,
 				maxPages: UISettings.miniTheme ? 1 : 5,
 				pageDots: !UISettings.miniTheme,
+				rowSelect: UISettings.rowSelect,
+				shortcuts: true,
 				fillFieldsCallback: fillFieldsCallback,
 				renderCellCallback: renderCellCallback,
-				updateInfoCallback: updateInfo
+				updateInfoCallback: updateInfo,
+				dragStartCallback: Refresher.pause,
+				dragEndCallback: dragEndCallback,
+				dragBlink: 'update'
 			});
 
 		$DownloadsTable.on('click', 'a', itemClick);
-		$DownloadsTable.on('click', UISettings.rowSelect ? 'tbody tr' : 'tbody div.check',
-			function(event) { $DownloadsTable.fasttable('itemCheckClick', UISettings.rowSelect ? this : this.parentNode.parentNode, event); });
-		$DownloadsTable.on('click', 'thead div.check',
-			function() { $DownloadsTable.fasttable('titleCheckClick') });
-		$DownloadsTable.on('mousedown', Util.disableShiftMouseDown);
+		$DownloadsTable.on('click', 'td:nth-child(2).dropdown-cell > div:not(.dropdown-disabled)', priorityClick);
+		$DownloadsTable.on('click', 'td:nth-child(3).dropdown-cell > div', statusClick);
+		$DownloadsTable.on('click', 'td:nth-child(5).dropdown-cell > div:not(.dropdown-disabled)', categoryClick);
+		$PriorityMenu.on('click', 'a', priorityMenuClick);
+		$CategoryMenu.on('click', 'a', categoryMenuClick);
+
+		DownloadsActionsMenu.init();
 	}
 
 	this.applyTheme = function()
 	{
+		this.redraw(true);
 		$DownloadsTable.fasttable('setPageSize', UISettings.read('DownloadsRecordsPerPage', 10),
 			UISettings.miniTheme ? 1 : 5, !UISettings.miniTheme);
 	}
 
 	this.update = function()
+	{
+		RPC.call('listgroups', [], groups_loaded, null, { prefer_cached: true });
+	}
+
+	function groups_loaded(_groups, _cached)
 	{
 		if (!groups)
 		{
 			$('#DownloadsTable_Category').css('width', DownloadsUI.calcCategoryColumnWidth());
 		}
-		
-		RPC.call('listgroups', [], groups_loaded);
-	}
 
-	function groups_loaded(_groups)
-	{
-		groups = _groups;
-		Downloads.groups = groups;
-		prepare();
+		cached = _cached;
+		if (!Refresher.isPaused() && !cached)
+		{
+			groups = _groups;
+			Downloads.groups = groups;
+			prepare();
+		}
 		RPC.next();
 	}
 
@@ -137,9 +154,18 @@ var Downloads = (new function($)
 		}
 	}
 
-	this.redraw = function()
+	this.redraw = function(force)
 	{
-		redraw_table();
+		if (cached && !force && lastDownloadRate === Status.status.DownloadRate)
+		{
+			return;
+		}
+
+		if (!Refresher.isPaused())
+		{
+			redraw_table();
+			lastDownloadRate = Status.status.DownloadRate;
+		}
 
 		Util.show($DownloadsTabBadge, groups.length > 0);
 		Util.show($DownloadsTabBadgeEmpty, groups.length === 0 && UISettings.miniTheme);
@@ -150,11 +176,11 @@ var Downloads = (new function($)
 	{
 		calcProgressLabels();
 	}
-	
+
 	/*** TABLE *************************************************************************/
 
 	var SEARCH_FIELDS = ['name', 'status', 'priority', 'category', 'estimated', 'age', 'size', 'remaining'];
-	
+
 	function redraw_table()
 	{
 		var data = [];
@@ -200,11 +226,11 @@ var Downloads = (new function($)
 		var group = item.data;
 
 		var status = DownloadsUI.buildStatus(group);
-		var priority = DownloadsUI.buildPriority(group.MaxPriority);
+		var priority = DownloadsUI.buildPriority(group);
 		var progresslabel = DownloadsUI.buildProgressLabel(group, nameColumnWidth);
 		var progress = DownloadsUI.buildProgress(group, item.data.size, item.data.left, item.data.estimated);
 		var dupe = DownloadsUI.buildDupe(group.DupeKey, group.DupeScore, group.DupeMode);
-		
+
 		var age = new Date().getTime() / 1000 - (group.MinPostTime + UISettings.timeZoneCorrection*60*60);
 		var propagation = '';
 		if (group.ActiveDownloads == 0 && age < parseInt(Options.option('PropagationDelay')) * 60)
@@ -213,42 +239,40 @@ var Downloads = (new function($)
 		}
 
 		var name = '<a href="#" data-nzbid="' + group.NZBID + '">' + Util.textToHtml(Util.formatNZBName(group.NZBName)) + '</a>';
+		name += DownloadsUI.buildEncryptedLabel(group.Parameters);
 
 		var url = '';
 		if (group.Kind === 'URL')
 		{
 			url = '<span class="label label-info">URL</span> ';
 		}
-		
+
 		var health = '';
 		if (group.Health < 1000 && (!group.postprocess ||
 			(group.Status === 'PP_QUEUED' && group.PostTotalTimeSec === 0)))
 		{
-			health = ' <span class="label ' + 
+			health = ' <span class="label ' +
 				(group.Health >= group.CriticalHealth ? 'label-warning' : 'label-important') +
 				'">health: ' + Math.floor(group.Health / 10) + '%</span> ';
 		}
 
-		var backup = '';
-		var backupPercent = calcBackupPercent(group);
-		if (backupPercent > 0)
-		{
-			backup = ' <a href="#" data-nzbid="' + group.NZBID + '" data-area="backup" class="badge-link"><span class="label label-warning" title="using backup news servers">backup: ' + 
-				(backupPercent < 10 ? Util.round1(backupPercent) : Util.round0(backupPercent)) + '%</span> ';
-		}
-		
-		var category = Util.textToHtml(group.Category);
+		var category = group.Category !== '' ? Util.textToHtml(group.Category) : '<span class="none-category">None</span>';
+		var backup = DownloadsUI.buildBackupLabel(group);
 
 		if (!UISettings.miniTheme)
 		{
-			var info = name + ' ' + url + priority + dupe + health + backup + propagation + progresslabel;
-			item.fields = ['<div class="check img-check"></div>', status, info, category, item.data.age, progress, item.data.estimated];
+			priority = '<div data-nzbid="' + group.NZBID + '"' +  (group.postprocess ? ' class="dropdown-disabled"' : '') + '>' + priority + '</div>';
+			status = '<div data-nzbid="' + group.NZBID + '">' + status + '</div>';
+			category = '<div data-nzbid="' + group.NZBID + '"' +  (group.postprocess ? ' class="dropdown-disabled"' : '') + '>' + category + '</div>';
+			var info = name + ' ' + url + dupe + health + backup + propagation + progresslabel;
+			item.fields = ['<div class="check img-check"></div>', priority, status, info, category, item.data.age, progress, item.data.estimated];
 		}
 		else
 		{
-			var info = '<div class="check img-check"></div><span class="row-title">' + name + '</span>' + url +
-				' ' + (group.Status === 'QUEUED' ? '' : status) + ' ' + priority + dupe + health + backup + propagation;
-			if (category)
+			var info = '<div class="check img-check"></div><span class="row-title">' +
+				name + '</span>' + url + ' ' + (group.MaxPriority == 0 ? '' : priority) +
+				' ' + (group.Status === 'QUEUED' ? '' : status) + dupe + health + backup + propagation;
+			if (group.Category !== '')
 			{
 				info += ' <span class="label label-status">' + category + '</span>';
 			}
@@ -263,51 +287,27 @@ var Downloads = (new function($)
 
 	function renderCellCallback(cell, index, item)
 	{
-		if (4 <= index && index <= 7)
+		if (index === 1)
+		{
+			cell.className = 'priority-cell' + (!UISettings.miniTheme ? ' dropdown-cell' : '');
+		}
+		else if (index === 2 || index === 4)
+		{
+			cell.className = !UISettings.miniTheme ? 'dropdown-cell dropafter-cell' : '';
+		}
+		else if (index === 3)
+		{
+			cell.className = !UISettings.miniTheme ? 'dropafter-cell' : '';
+		}
+		else if (index === 5)
+		{
+			cell.className = 'text-right' + (!UISettings.miniTheme ? ' dropafter-cell' : '');
+		}
+		else if (6 <= index && index <= 8)
 		{
 			cell.className = 'text-right';
 		}
 	}
-	
-	function calcBackupPercent(group)
-	{
-		var downloadedArticles = group.SuccessArticles + group.FailedArticles;
-		if (downloadedArticles === 0)
-		{
-			return 0;
-		}
-		
-		if (minLevel === null)
-		{
-			for (var i=0; i < Status.status.NewsServers.length; i++)
-			{
-				var server = Status.status.NewsServers[i];
-				var level = parseInt(Options.option('Server' + server.ID + '.Level'));
-				if (minLevel === null || minLevel > level)
-				{
-					minLevel = level;
-				}
-			}	
-		}
-		
-		var backupArticles = 0;
-		for (var j=0; j < group.ServerStats.length; j++)
-		{
-			var stat = group.ServerStats[j];
-			var level = parseInt(Options.option('Server' + stat.ServerID + '.Level'));
-			if (level > minLevel && stat.SuccessArticles > 0)
-			{
-				backupArticles += stat.SuccessArticles;
-			}
-		}
-
-		var backupPercent = 0;
-		if (backupArticles > 0)
-		{
-			backupPercent = backupArticles * 100.0 / downloadedArticles;
-		}
-		return backupPercent;
-	}
 
 	this.recordsPerPageChange = function()
 	{
@@ -337,9 +337,40 @@ var Downloads = (new function($)
 		progressLabels.css('max-width', nameColumnWidth);
 		progressLabels.show();
 	}
-	
+
+	this.processShortcut = function(key)
+	{
+		switch (key)
+		{
+			case 'A': Upload.addClick(); return true;
+			case 'D': case 'Delete': case 'Meta+Backspace': Downloads.deleteClick(); return true;
+			case 'E': case 'Enter': Downloads.editClick(); return true;
+			case 'U': Downloads.moveClick('up'); return true;
+			case 'N': Downloads.moveClick('down'); return true;
+			case 'T': Downloads.moveClick('top'); return true;
+			case 'B': Downloads.moveClick('bottom'); return true;
+			case 'P': Downloads.pauseClick(); return true;
+			case 'R': Downloads.resumeClick(); return true;
+			case 'M': Downloads.mergeClick(); return true;
+		}
+		return $DownloadsTable.fasttable('processShortcut', key);
+	}
+
 	/*** EDIT ******************************************************/
 
+	function findGroup(nzbid)
+	{
+		for (var i=0; i<groups.length; i++)
+		{
+			var gr = groups[i];
+			if (gr.NZBID == nzbid)
+			{
+				return gr;
+			}
+		}
+		return null;
+	}
+
 	function itemClick(e)
 	{
 		e.preventDefault();
@@ -355,7 +386,7 @@ var Downloads = (new function($)
 		Refresher.update();
 		if (notification)
 		{
-			Notification.show(notification);
+			PopupNotification.show(notification);
 			notification = null;
 		}
 	}
@@ -375,12 +406,12 @@ var Downloads = (new function($)
 			{
 				if (group.postprocess && !allowPostProcess)
 				{
-					Notification.show('#Notif_Downloads_CheckPostProcess');
+					PopupNotification.show('#Notif_Downloads_CheckPostProcess');
 					return null;
 				}
 				if (group.Kind === 'URL' && !allowUrl)
 				{
-					Notification.show('#Notif_Downloads_CheckURL');
+					PopupNotification.show('#Notif_Downloads_CheckURL');
 					return null;
 				}
 
@@ -390,13 +421,24 @@ var Downloads = (new function($)
 
 		if (checkedEditIDs.length === 0 && !allowEmpty)
 		{
-			Notification.show('#Notif_Downloads_Select');
+			PopupNotification.show('#Notif_Downloads_Select');
 			return null;
 		}
 
 		return checkedEditIDs;
 	}
 
+	function buildContextIdList(group)
+	{
+		var editIds = checkBuildEditIDList(true, true, true);
+		if (editIds.indexOf(group.NZBID) === -1)
+		{
+			editIds = [group.NZBID];
+		}
+		return editIds;
+	}
+	this.buildContextIdList = buildContextIdList;
+
 	/*** TOOLBAR: SELECTED ITEMS ******************************************************/
 
 	this.editClick = function()
@@ -427,7 +469,7 @@ var Downloads = (new function($)
 
 		if (checkedEditIDs.length < 2)
 		{
-			Notification.show('#Notif_Downloads_SelectMulti');
+			PopupNotification.show('#Notif_Downloads_SelectMulti');
 			return;
 		}
 
@@ -442,7 +484,7 @@ var Downloads = (new function($)
 			return;
 		}
 		notification = '#Notif_Downloads_Paused';
-		RPC.call('editqueue', ['GroupPause', 0, '', checkedEditIDs], editCompleted);
+		RPC.call('editqueue', ['GroupPause', '', checkedEditIDs], editCompleted);
 	}
 
 	this.resumeClick = function()
@@ -453,7 +495,7 @@ var Downloads = (new function($)
 			return;
 		}
 		notification = '#Notif_Downloads_Resumed';
-		RPC.call('editqueue', ['GroupResume', 0, '', checkedEditIDs], function()
+		RPC.call('editqueue', ['GroupResume', '', checkedEditIDs], function()
 		{
 			if (Options.option('ParCheck') === 'force')
 			{
@@ -461,7 +503,7 @@ var Downloads = (new function($)
 			}
 			else
 			{
-				RPC.call('editqueue', ['GroupPauseExtraPars', 0, '', checkedEditIDs], editCompleted);
+				RPC.call('editqueue', ['GroupPauseExtraPars', '', checkedEditIDs], editCompleted);
 			}
 		});
 	}
@@ -490,7 +532,7 @@ var Downloads = (new function($)
 
 		if (downloadIDs.length === 0 && postprocessIDs.length === 0)
 		{
-			Notification.show('#Notif_Downloads_Select');
+			PopupNotification.show('#Notif_Downloads_Select');
 			return;
 		}
 
@@ -500,7 +542,7 @@ var Downloads = (new function($)
 		{
 			if (postprocessIDs.length > 0)
 			{
-				RPC.call('editqueue', ['PostDelete', 0, '', postprocessIDs], editCompleted);
+				RPC.call('editqueue', ['PostDelete', '', postprocessIDs], editCompleted);
 			}
 			else
 			{
@@ -512,7 +554,7 @@ var Downloads = (new function($)
 		{
 			if (downloadIDs.length > 0)
 			{
-				RPC.call('editqueue', [command, 0, '', downloadIDs], deletePosts);
+				RPC.call('editqueue', [command, '', downloadIDs], deletePosts);
 			}
 			else
 			{
@@ -554,15 +596,100 @@ var Downloads = (new function($)
 		}
 
 		notification = '';
-		RPC.call('editqueue', [EditAction, EditOffset, '', checkedEditIDs], editCompleted);
+		RPC.call('editqueue', [EditAction, '' + EditOffset, checkedEditIDs], editCompleted);
 	}
 
-	this.sort = function(order)
+	this.sort = function(order, e)
 	{
+		e.preventDefault();
+		e.stopPropagation();
 		var checkedEditIDs = checkBuildEditIDList(true, true, true);
 		notification = '#Notif_Downloads_Sorted';
-		RPC.call('editqueue', ['GroupSort', 0, order, checkedEditIDs], editCompleted);
+		RPC.call('editqueue', ['GroupSort', order, checkedEditIDs], editCompleted);
 	}
+
+	function dragEndCallback(info)
+	{
+		if (info)
+		{
+			RPC.call('editqueue', [info.direction === 'after' ? 'GroupMoveAfter' : 'GroupMoveBefore',
+				'' + info.position, info.ids], function(){ Refresher.resume(true); });
+		}
+		else
+		{
+			Refresher.resume();
+		}
+	}
+
+	function priorityClick(e)
+	{
+		e.preventDefault();
+		e.stopPropagation();
+		var group = findGroup($(this).attr('data-nzbid'));
+		var editIds = buildContextIdList(group);
+		$PriorityMenu.data('nzbids', editIds);
+		DownloadsUI.updateContextWarning($PriorityMenu, editIds);
+		$('i', $PriorityMenu).removeClass('icon-ok').addClass('icon-empty');
+		$('li[data=' + group.MaxPriority + '] i', $PriorityMenu).addClass('icon-ok');
+		Frontend.showPopupMenu($PriorityMenu, 'left',
+			{ left: $(this).offset().left - 30, top: $(this).offset().top,
+				width: $(this).width() + 30, height: $(this).outerHeight() - 2});
+	}
+
+	function priorityMenuClick(e)
+	{
+		e.preventDefault();
+		var priority = $(this).parent().attr('data');
+		var nzbids = $PriorityMenu.data('nzbids');
+		notification = '#Notif_Downloads_Changed';
+		RPC.call('editqueue', ['GroupSetPriority', '' + priority, nzbids], editCompleted);
+	}
+
+	function statusClick(e)
+	{
+		e.preventDefault();
+		e.stopPropagation();
+		var group = findGroup($(this).attr('data-nzbid'));
+
+		DownloadsActionsMenu.showPopupMenu(group, 'left',
+			{ left: $(this).offset().left - 30, top: $(this).offset().top,
+				width: $(this).width() + 30, height: $(this).outerHeight() - 2 },
+			function(_notification) { notification = _notification; },
+			editCompleted);
+	}
+
+	function categoryClick(e)
+	{
+		e.preventDefault();
+		e.stopPropagation();
+		DownloadsUI.fillCategoryMenu($CategoryMenu);
+		var group = findGroup($(this).attr('data-nzbid'));
+
+		if (group.postprocess)
+		{
+			return;
+		}
+
+		var editIds = buildContextIdList(group);
+		$CategoryMenu.data('nzbids', editIds);
+		DownloadsUI.updateContextWarning($CategoryMenu, editIds);
+		$('i', $CategoryMenu).removeClass('icon-ok').addClass('icon-empty');
+		$('li[data="' + group.Category + '"] i', $CategoryMenu).addClass('icon-ok');
+
+		Frontend.showPopupMenu($CategoryMenu, 'left',
+			{ left: $(this).offset().left - 30, top: $(this).offset().top,
+				width: $(this).width() + 30, height: $(this).outerHeight() - 2 });
+	}
+
+	function categoryMenuClick(e)
+	{
+		e.preventDefault();
+		var category = $(this).parent().attr('data');
+		var nzbids = $CategoryMenu.data('nzbids');
+		notification = '#Notif_Downloads_Changed';
+		RPC.call('editqueue', ['GroupApplyCategory', category, nzbids], editCompleted);
+	}
+
 }(jQuery));
 
 
@@ -571,11 +698,12 @@ var Downloads = (new function($)
 var DownloadsUI = (new function($)
 {
 	'use strict';
-	
+
 	// State
 	var categoryColumnWidth = null;
 	var dupeCheck = null;
-	
+	var minLevel = null;
+
 	this.fillPriorityCombo = function(combo)
 	{
 		combo.empty();
@@ -598,6 +726,23 @@ var DownloadsUI = (new function($)
 		}
 	}
 
+	this.fillCategoryMenu = function(menu)
+	{
+		if (menu.data('initialized'))
+		{
+			return;
+		}
+		var templ = $('li:last-child', menu);
+		for (var i=0; i < Options.categories.length; i++)
+		{
+			var item = templ.clone().show();
+			$('td:last-child', item).text(Options.categories[i]);
+			item.attr('data', Options.categories[i]);
+			menu.append(item);
+		}
+		menu.data('initialized', true);
+	}
+
 	this.buildStatusText = function(group)
 	{
 		var statusText = Downloads.statusData[group.Status].Text;
@@ -607,12 +752,12 @@ var DownloadsUI = (new function($)
 		}
 		return statusText;
 	}
-		
+
 	this.buildStatus = function(group)
 	{
 		var statusText = Downloads.statusData[group.Status].Text;
 		var badgeClass = '';
-		
+
 		if (group.postprocess && group.Status !== 'PP_QUEUED' && group.Status !== 'QS_QUEUED')
 		{
 			badgeClass = Status.status.PostPaused && group.MinPriority < 900 ? 'label-warning' : 'label-success';
@@ -630,7 +775,7 @@ var DownloadsUI = (new function($)
 			statusText = 'INTERNAL_ERROR (' + group.Status + ')';
 			badgeClass = 'label-important';
 		}
-		
+
 		return '<span class="label label-status ' + badgeClass + '">' + statusText + '</span>';
 	}
 
@@ -662,7 +807,7 @@ var DownloadsUI = (new function($)
 			remaining = '';
 			percent = Math.round(group.PostStageProgress / 10);
 		}
-		
+
 		if (group.Kind === 'URL')
 		{
 			totalsize = '';
@@ -749,27 +894,94 @@ var DownloadsUI = (new function($)
 		}
 	}
 
-	this.buildPriority = function(priority)
+	this.buildPriority = function(group)
 	{
-		switch (priority)
+		var priority = group.MaxPriority
+		var text;
+
+		if (priority >= 900) text = ' <div class="icon-circle-red" title="Force priority"></div>';
+		else if (priority > 50) text = ' <div class="icon-ring-fill-red" title="Very high priority"></div>';
+		else if (priority > 0) text = ' <div class="icon-ring-red" title="High priority"></div>';
+		else if (priority == 0) text = ' <div class="icon-ring-ltgrey" title="Normal priority"></div>';
+		else if (priority >= -50) text = ' <div class="icon-ring-blue" title="Low priority"></div>';
+		else text = ' <div class="icon-ring-fill-blue" title="Very low priority"></div>';
+
+		if ([900, 100, 50, 0, -50, -100].indexOf(priority) == -1)
 		{
-			case 0: return '';
-			case 900: return ' <span class="label label-priority label-important">force priority</span>';
-			case 100: return ' <span class="label label-priority label-important">very high priority</span>';
-			case 50: return ' <span class="label label-priority label-important">high priority</span>';
-			case -50: return ' <span class="label label-priority label-info">low priority</span>';
-			case -100: return ' <span class="label label-priority label-info">very low priority</span>';
-		}
-		if (priority > 0)
-		{
-			return ' <span class="label label-priority label-important">priority: ' + priority + '</span>';
-		}
-		else if (priority < 0)
-		{
-			return ' <span class="label label-priority label-info">priority: ' + priority + '</span>';
+			text = text.replace('priority', 'priority (' + priority + ')');
 		}
+
+		return text;
 	}
-	
+
+	this.buildEncryptedLabel = function(parameters)
+	{
+		var encryptedPassword = '';
+
+		for (var i = 0; i < parameters.length; i++)
+		{
+			if (parameters[i]['Name'].toLowerCase() === '*unpack:password')
+			{
+				encryptedPassword = parameters[i]['Value'];
+				break;
+			}
+		}
+		return encryptedPassword != '' ?
+			' <span class="label label-info" title="'+ Util.textToAttr(encryptedPassword) +'">encrypted</span>' : '';
+	}
+
+	this.buildBackupLabel = function(group)
+	{
+		var backup = '';
+		var backupPercent = calcBackupPercent(group);
+		if (backupPercent > 0)
+		{
+			backup = ' <a href="#" data-nzbid="' + group.NZBID + '" data-area="backup" class="badge-link"><span class="label label-warning" title="with backup news servers">backup: ' +
+				(backupPercent < 10 ? Util.round1(backupPercent) : Util.round0(backupPercent)) + '%</span> ';
+		}
+		return backup;
+	}
+
+	function calcBackupPercent(group)
+	{
+		var downloadedArticles = group.SuccessArticles + group.FailedArticles;
+		if (downloadedArticles === 0)
+		{
+			return 0;
+		}
+
+		if (minLevel === null)
+		{
+			for (var i=0; i < Status.status.NewsServers.length; i++)
+			{
+				var server = Status.status.NewsServers[i];
+				var level = parseInt(Options.option('Server' + server.ID + '.Level'));
+				if (minLevel === null || minLevel > level)
+				{
+					minLevel = level;
+				}
+			}
+		}
+
+		var backupArticles = 0;
+		for (var j=0; j < group.ServerStats.length; j++)
+		{
+			var stat = group.ServerStats[j];
+			var level = parseInt(Options.option('Server' + stat.ServerID + '.Level'));
+			if (level > minLevel && stat.SuccessArticles > 0)
+			{
+				backupArticles += stat.SuccessArticles;
+			}
+		}
+
+		var backupPercent = 0;
+		if (backupArticles > 0)
+		{
+			backupPercent = backupArticles * 100.0 / downloadedArticles;
+		}
+		return backupPercent;
+	}
+
 	function formatDupeText(dupeKey, dupeScore, dupeMode)
 	{
 		dupeKey = dupeKey.replace('rageid=', '');
@@ -845,15 +1057,39 @@ var DownloadsUI = (new function($)
 				var catWidth = widthHelper.width();
 				categoryColumnWidth = Math.max(categoryColumnWidth, catWidth);
 			}
-						
+
 			widthHelper.remove();
-			
-			categoryColumnWidth += 'px';
+
+			categoryColumnWidth = (categoryColumnWidth + 8) + 'px';
 		}
 
 		return categoryColumnWidth;
 	}
 
+	this.buildDNZBLinks = function(parameters, prefix)
+	{
+		$('.' + prefix).hide();
+		var hasItems = false;
+
+		for (var i=0; i < parameters.length; i++)
+		{
+			var param = parameters[i];
+			if (param.Name.substr(0, 6) === '*DNZB:')
+			{
+				var linkName = param.Name.substr(6, 100);
+				var $paramLink = $('#' + prefix + '_' + linkName);
+				if($paramLink.length > 0)
+				{
+					$paramLink.attr('href', param.Value);
+					$paramLink.show();
+					hasItems = true;
+				}
+			}
+		}
+
+		Util.show('#' + prefix + '_Section', hasItems);
+	}
+
 	this.deleteConfirm = function(actionCallback, multi, hasNzb, hasUrl, selCount)
 	{
 		var dupeCheck = Options.option('DupeCheck') === 'yes';
@@ -893,4 +1129,105 @@ var DownloadsUI = (new function($)
 
 		ConfirmDialog.showModal('DownloadsDeleteConfirmDialog', action, init, selCount);
 	}
+
+	this.updateContextWarning = function(menu, editIds)
+	{
+		var warning = $('.dropdown-warning', $(menu));
+		Util.show(warning, editIds.length > 1);
+		warning.text(editIds.length + ' records selected');
+	}	
+}(jQuery));
+
+/*** DOWNLOADS ACTION MENU *************************************************************************/
+
+var DownloadsActionsMenu = (new function()
+{
+	'use strict'
+
+	var $ActionsMenu;
+	var curGroup;
+	var beforeCallback;
+	var completedCallback;
+	var editIds;
+
+	this.init = function()
+	{
+		$ActionsMenu = $('#DownloadsActionsMenu');
+		$('#DownloadsActions_Pause').click(itemPause);
+		$('#DownloadsActions_Resume').click(itemResume);
+		$('#DownloadsActions_Delete').click(itemDelete);
+		$('#DownloadsActions_CancelPP').click(itemCancelPP);
+	}
+
+	this.showPopupMenu = function(group, anchor, rect, before, completed)
+	{
+		curGroup = group;
+		beforeCallback = before;
+		completedCallback = completed;
+		editIds = Downloads.buildContextIdList(group);
+
+		// setup menu items
+		Util.show('#DownloadsActions_CancelPP', group.postprocess);
+		Util.show('#DownloadsActions_Delete', !group.postprocess);
+		Util.show('#DownloadsActions_Pause', group.Kind === 'NZB' && !group.postprocess);
+		Util.show('#DownloadsActions_Resume', false);
+		DownloadsUI.updateContextWarning($ActionsMenu, editIds);
+
+		if (!group.postprocess &&
+			(group.RemainingSizeHi == group.PausedSizeHi &&
+				group.RemainingSizeLo == group.PausedSizeLo &&
+				group.Kind === 'NZB'))
+		{
+			$('#DownloadsActions_Resume').show();
+			$('#DownloadsActions_Pause').hide();
+		}
+
+		DownloadsUI.buildDNZBLinks(group.Parameters, 'DownloadsActions_DNZB');
+	
+		Frontend.showPopupMenu($ActionsMenu, anchor, rect);
+	}
+
+	function itemPause(e)
+	{
+		e.preventDefault();
+		beforeCallback('#Notif_Downloads_Paused');
+		RPC.call('editqueue', ['GroupPause', '', editIds], completedCallback);
+	}
+
+	function itemResume(e)
+	{
+		e.preventDefault();
+		beforeCallback('#Notif_Downloads_Resumed');
+		RPC.call('editqueue', ['GroupResume', '', editIds], function()
+		{
+			if (Options.option('ParCheck') === 'force')
+			{
+				completedCallback();
+			}
+			else
+			{
+				RPC.call('editqueue', ['GroupPauseExtraPars', '', editIds], completedCallback);
+			}
+		});
+	}
+
+	function itemDelete(e)
+	{
+		e.preventDefault();
+		DownloadsUI.deleteConfirm(doItemDelete, false, curGroup.Kind === 'NZB', curGroup.Kind === 'URL');
+	}
+
+	function doItemDelete(command)
+	{
+		beforeCallback('#Notif_Downloads_Deleted');
+		RPC.call('editqueue', [command, '', editIds], completedCallback);
+	}
+
+	function itemCancelPP(e)
+	{
+		e.preventDefault();
+		beforeCallback('#Notif_Downloads_PostCanceled');
+		RPC.call('editqueue', ['PostDelete', '', editIds], completedCallback);
+	}
+
 }(jQuery));
